@@ -10,7 +10,7 @@ substdio out = SUBSTDIO_FDBUF(write,1,buf,sizeof(buf));
 void main()
 {
  char ch;
- ch = LSPAWN_BREAK;
+ ch = USEREXT_BREAK;
  substdio_put(&out,&ch,1);
  substdio_puts(&out,"\n");
  substdio_flush(&out);
